/**
 * EasyBeans
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: KnopflerfishFramework.java 1834 2007-10-04 16:04:18Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.osgi.plugins.launcher.knopflerfish;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.Enumeration;
import java.util.Properties;

import org.knopflerfish.framework.Framework;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleException;
import org.ow2.easybeans.osgi.plugins.launcher.api.FrameworkException;
import org.ow2.easybeans.osgi.plugins.launcher.api.IFramework;
import org.ow2.util.file.FileUtils;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Implementation of the launcher for Knopflerfish.
 * @author Florent BENOIT
 */
public class KnopflerfishFramework implements IFramework {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(KnopflerfishFramework.class);

    /**
     * Link to the Knopflerfish embedded object.
     */
    private Framework knopflerfish = null;

    /**
     * Working directory.
     */
    private File workingDirectory = null;


    /**
     * Configure the Knopflerfish gateway by using the given configuration file.
     * @param config the URL of the configuration file to use
     * @throws FrameworkException if the configuration is not done
     */
    public void configure(final URL config) throws FrameworkException {
            // No config, do nothing
            if (config == null) {
                return;
            }

            // Open a connection on the given URL
            URLConnection urlConnection;
            try {
                urlConnection = config.openConnection();
            } catch (IOException e) {
                throw new FrameworkException("Cannot open the connection on the URL '" + config + "'.", e);
            }
            urlConnection.setDefaultUseCaches(false);

            // Populate a properties object with the stream of the URL
            Properties prop = new Properties();
            InputStream is = null;
            try {
                is = urlConnection.getInputStream();
                prop.load(is);
            } catch (IOException e) {
                throw new FrameworkException("Cannot open the connection on the URL '" + config + "'.", e);
            } finally {
                if (is != null) {
                    try {
                        is.close();
                    } catch (IOException e) {
                        logger.error("Cannot close the stream", e);
                    }
                }
            }

            Enumeration<Object> enumeration = prop.keys();
            while (enumeration.hasMoreElements()) {
                String key = enumeration.nextElement().toString();
                String value = prop.get(key).toString();
                System.setProperty(key, value);
            }

        }

    /**
     * Start the framework.
     * @throws FrameworkException if the framework can't be started.
     */
    public void start() throws FrameworkException {

        // Cleanup
        if (workingDirectory.exists()) {
            FileUtils.delete(workingDirectory);
        }

        // Set System properties
        System.setProperty("org.osgi.framework.dir", workingDirectory.getAbsolutePath());
        System.setProperty("org.knopflerfish.framework.bundlestorage", "file");


        // Create Knopflerfish instance
        try {
            knopflerfish = new Framework(null);
        } catch (Exception e) {
            throw new FrameworkException("Cannot create Knopflerfish instance", e);
        }


        // Start it
        try {
            knopflerfish.launch(0);
        } catch (BundleException e) {
            throw new FrameworkException("Cannot start Knopflerfish", e);
        }
    }


    /**
     * Stop the framework.
     * @throws FrameworkException if the framework can't be stopped.
     */
    public void stop() throws FrameworkException {

        // Cannot stop unknown Knopflerfish object
        if (knopflerfish == null) {
            throw new FrameworkException("No Knopflerfish object configured");
        }

        // Stop Knopflerfish instance
        try {
            knopflerfish.shutdown();
        } catch (Exception e) {
            throw new FrameworkException("Cannot stop Knopflerfish", e);
        }
    }

    /**
     * Gets the bundle context of the framework.
     * @return the bundle context allowing to do some operations.
     */
    public BundleContext getBundleContext() {
        return knopflerfish.getSystemBundleContext();
    }

    /**
     * Checks if the framework is running.
     * @return true if the framework is running
     */
    public boolean isRunning() {
        return getBundleContext().getBundle().getState() == Bundle.ACTIVE;
    }


    /**
     * Sets the working directory.
     * @param workingDirectory the given directory used to store some files.
     */
    public void setWorkingDirectory(final File workingDirectory) {
        this.workingDirectory = workingDirectory;
    }

}

